

puts "loading sys.ERDA.tcl"

variable registerlist

lappend registerlist [list NetVar.ERDA_NumSamples  "-n ERDA_NumSamples -t int"]
lappend registerlist [list NetVar.ERDA_VMERetries  "-n ERDA_VMERetries -t int"]
lappend registerlist [list NetVar.ERDA_BERREnable  "-n ERDA_BERREnable -t int"]
lappend registerlist [list NetVar.ERDA_VMEBufSize  "-n ERDA_VMEBufSize -t int"]


proc Set_ERDA_Options {} {puts "Set_ERDA_Options"
    variable Options

    LoadOptions

   if {![info exists Options(ERDA_MaxEvtsPerBuf)]}         {set Options(ERDA_MaxEvtsPerBuf) 1}
   if {![info exists Options(ERDA_MaxBLT)]}                {set Options(ERDA_MaxBLT) 1}
   if {![info exists Options(ERDA_MemBufSize)]}            {set Options(ERDA_MemBufSize) 1024}
   if {![info exists Options(ERDA_VMEBufSize)]}            {set Options(ERDA_VMEBufSize) 16384}

   if {![info exists Options(ERDA_channels)]}   {set Options(ERDA_channels) 4096}
   if {![info exists Options(ERDA_offset)]}     {set Options(ERDA_offset) 0}
   if {![info exists Options(ERDA_shift)]}      {set Options(ERDA_shift) 0}

   if {![info exists Options(TAC_HistEnable)]} {set Options(TAC_HistEnable) 1}
   if {![info exists Options(ERDA_HistEnable)]} {set Options(ERDA_HistEnable) 1}
   if {![info exists Options(2D_DATA_HistEnable)]} {set Options(2D_DATA_HistEnable) 1}
   if {![info exists Options(ERDA_2D_HistEnable)]} {set Options(ERDA_2D_HistEnable) 1}

   if {![info exists Options(ERDA_GroupBase)]}  {set Options(ERDA_GroupBase) 1}   ;#  default Group Base

    SaveOptions
}

proc ERDA_Configure {} {puts "ERDA_Configure"
    variable Options

#    update_Options
}

proc Dump_Raw_ERDA_Data {data} {
     
     set info ""

     foreach word $data {
         if {[string range $word 0 1] != "0x"} then {set word 0x$word}
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         set type [expr $word >> 24 & 7]
         switch $type {
            0 {
             set event_type "data"
             set channel [expr $word >> 16 & 0x3f]
             set un [expr $word >> 13 & 1]
             set ov [expr $word >> 12 & 1]
             set adc [expr $word  & 0xfff]
             append info "[format "0x%8x    %8s   channel= %5i; un= %3i; ov= %3i; data= 0x%5x" $word $event_type $channel $un $ov $adc]"
            }
            2 {
             set event_type "header"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word >> 8 & 0x3f]
             append info "[format "0x%8x    %8s   module= %5i; channels= %6i" $word $event_type $geo $counter]"
            }
            4 {
             set event_type "trailer"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word & 0xffffff]
             append info "[format "0x%8x    %8s   module= %5i; event data counter= %10i" $word $event_type $geo $counter]"
            }
            default {
             append info "$word - not valid datum\n"
            }
         }
         append info "\n"
     }
   return $info
}


proc ERDA_DAQ-go {} {puts "ERDA_DAQ-go"

}

proc ERDA_DAQ-halt {} {puts "ERDA_DAQ-halt"}

proc ERDA_DAQ-setup {} {puts "ERDA_DAQ-setup"

      ERDA_Configure

}

proc ERDA_DAQ-flush {} {puts "ERDA_DAQ-flush"

}

proc ERDA_DAQ-reset {} {puts "ERDA_DAQ-reset"}

proc define_ERDA_config {} {puts "define_ERDA_config"

}


proc create_ERDA_spectra {} {puts "create_ERDA_spectra"
    variable Options 

         if {$Options(TAC_HistEnable) > 0} {

#    histograms for TAC (1D)

           set spectrumname TAC
           set z [catch {CreateSpectrum1D $spectrumname 4096} m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              puts "Attempting to create spectrum $spectrumname"
              error $m
           } else {
              set group 200
              map_spectrum  "$spectrumname $group 0 0 0"
           }
         }

         if {$Options(ERDA_2D_HistEnable) > 0} {

#    histograms for 2D erda

           set item 1
           foreach hist [list 2d.erda1 2d.erda2 2d.erda3] {

              set spectrumname $hist
              set z [catch {CreateSpectrum2D $spectrumname 183 93} m]

#    Allow "Spectrum already exists" so that further spectra can be created
              if {$z!=0 && $m==0x30004} then {set z 0}
              if {$z!=0} then {
                 puts "Attempting to create spectrum $spectrumname"
                 error $m
              } else {
                 set group 252
                 map_spectrum  "$spectrumname $group $item 0 0"
              }
              set label "MEIS 2D ERDA Window $item - Run * - ***"
              set z [catch {SpectrumPutTitle $spectrumname $label} m]
              incr item
           }
         }

         if {$Options(2D_DATA_HistEnable) > 0} {

#    histograms for 2D data

           set item 3
           foreach hist [list 2d.full 2d.data] {

              set spectrumname $hist
              if {$item == 3} {set z [catch {CreateSpectrum2D $spectrumname 256 1023} m]}
              if {$item == 4} {set z [catch {CreateSpectrum2D $spectrumname 183 93} m]}

#    Allow "Spectrum already exists" so that further spectra can be created
              if {$z!=0 && $m==0x30004} then {set z 0}
              if {$z!=0} then {
                 puts "Attempting to create spectrum $spectrumname"
                 error $m
              } else {
                 set group 253
                 map_spectrum  "$spectrumname $group $item 0 0"
              }
              incr item
           }
         } 

         if {$Options(ERDA_HistEnable) > 0} {

#    histograms for erda

           set item 1
           foreach hist [list angle energy] {

              set spectrumname $hist
              if {$item == 1} {set z [catch {CreateSpectrum1D $spectrumname 256} m]}
              if {$item == 2} {set z [catch {CreateSpectrum1D $spectrumname 1023} m]}

#    Allow "Spectrum already exists" so that further spectra can be created
              if {$z!=0 && $m==0x30004} then {set z 0}
              if {$z!=0} then {
                 puts "Attempting to create spectrum $spectrumname"
                 error $m
              } else {
                 set group 253
                 map_spectrum  "$spectrumname $group $item 0 0"
              }
              if {$item == 1} {set label "MEIS Angle  - Run * - ***"}
              if {$item == 2} {set label "MEIS Energy  - Run * - ***"}
              set z [catch {SpectrumPutTitle $spectrumname $label} m]
              incr item
           }

           set item 1
           foreach hist [list scan.tilt scan.spin scan.rotation scan.height] {

              set spectrumname $hist
              set z [catch {CreateSpectrum1D $spectrumname 100} m]

#    Allow "Spectrum already exists" so that further spectra can be created
              if {$z!=0 && $m==0x30004} then {set z 0}
              if {$z!=0} then {
                 puts "Attempting to create spectrum $spectrumname"
                 error $m
              } else {
                 set group 100
                 map_spectrum  "$spectrumname $group $item 0 0"
              }
              incr item
           }
         }
}


   proc CreateSpectrum2D {Name Range1 Range2} {
       global errorCode errorInfo
       variable Wmsg

       set z [catch {::namespace inscope ::urn:SpectrumService SpecNew2D  $Name 0 $Range1 0 $Range2} Wmsg]

       if {$z == 0} {
#           puts "SpecNew2D returned: [lindex $Wmsg 5]"
           set errorCode 0
       } else {
           puts "SpecNew2D failed: Code= $errorCode; Info= [first [split $errorInfo \n]]"
       }
       return $errorCode
   }



